#include <string.h>
#include "codepages.h"
#include "rucode.h"

void rucode_init() {
    rucode_codepages[0].name = rucode_codepage_koi8r_name;
    rucode_codepages[0].letters = rucode_codepage_koi8r_letters;
    rucode_codepages[0].num_quotes = RUCODE_CODEPAGE_KOI8R_NUM_QUOTES;
    rucode_codepages[0].quotes = rucode_codepage_koi8r_quotes;

    rucode_codepages[1].name = rucode_codepage_windows1251_name;
    rucode_codepages[1].letters = rucode_codepage_windows1251_letters;
    rucode_codepages[1].num_quotes = RUCODE_CODEPAGE_WINDOWS1251_NUM_QUOTES;
    rucode_codepages[1].quotes = rucode_codepage_windows1251_quotes;

    rucode_codepages[2].name = rucode_codepage_cp866_name;
    rucode_codepages[2].letters = rucode_codepage_cp866_letters;
    rucode_codepages[2].num_quotes = RUCODE_CODEPAGE_CP866_NUM_QUOTES;
    rucode_codepages[2].quotes = rucode_codepage_cp866_quotes;
}

int rucode_codepage_name_to_id(char *name) {
    int i;
    
    for (i = 0; i < RUCODE_NUM_CODEPAGES; i++) {
        if (!strcmp(rucode_codepages[i].name, name)) {
            return i;
        }
    }

    return -1;
}

int rucode_letter_to_index(int codepage, char letter) {
    int i;

    for (i = 0; i < RUCODE_NUM_LETTERS * 2; i++) {
        if (rucode_codepages[codepage].letters[i] == letter) {
            return i;
        }
    }

    return -1;
}

int rucode_is_quote(int codepage, char letter) {
    int i;
    
    for (i = 0; i < rucode_codepages[codepage].num_quotes; i++) {
        if (rucode_codepages[codepage].quotes[i] == letter) {
            return 1;
        }
    }

    return 0;
}

void rucode_strip_quotes(char *buffer, int len, int codepage) {
    int i;
    
    if (rucode_codepages[codepage].num_quotes != 0) {
        for (i = 0; i < len; i++) {
            if (rucode_is_quote(codepage, buffer[i])) {
                buffer[i] = '"';
            }
        }
    }
 }

int rucode_score(char *buffer, int len, int codepage) {
    int i;
    int index;
    int num_total;
    int num_letters;
    int num_big;

    num_total = 0;
    num_letters = 0;
    num_big = 0;
    
    for (i = 0; i < len; i++) {
        if ((unsigned char)buffer[i] >= 0x80) {
            num_total++;
        }
        
        index = rucode_letter_to_index(codepage, buffer[i]);
        if (index != -1) {
            num_letters++;
            if (index >= RUCODE_NUM_LETTERS) {
                num_big++; 
            }
        }
    }

    if (num_total == 0) {
        return 100;
    }

    return ((num_letters - num_big) * 100) / num_total;
}

int rucode_guess_codepage(char *buffer, int len) {
    int cur_codepage;
    int best_codepage;
    int cur_score;
    int best_score;

    for (cur_codepage = 0; cur_codepage < RUCODE_NUM_CODEPAGES; cur_codepage++) {
        cur_score = rucode_score(buffer, len, cur_codepage);
        if (cur_codepage == 0 || cur_score > best_score) {
            best_codepage = cur_codepage;
            best_score = cur_score;
        }
    }

    return best_codepage;
}

void rucode_do(char *buffer, int len, int from_codepage, int to_codepage) {
    int i;
    int index;

    rucode_strip_quotes(buffer, len, from_codepage);
    
    for (i = 0; i < len; i++) {
        index = rucode_letter_to_index(from_codepage, buffer[i]);
        if (index != -1) {
            buffer[i] = rucode_codepages[to_codepage].letters[index];
        }
    }
}
